/**************************************************************************
***    
*** Copyright (c) 1995-2000 Regents of the University of California,
***               Andrew E. Caldwell, Andrew B. Kahng and Igor L. Markov
***
***  Contact author(s): abk@cs.ucla.edu, imarkov@cs.ucla.edu
***  Affiliation:   UCLA, Computer Science Department,
***                 Los Angeles, CA 90095-1596 USA
***
***  Permission is hereby granted, free of charge, to any person obtaining 
***  a copy of this software and associated documentation files (the
***  "Software"), to deal in the Software without restriction, including
***  without limitation 
***  the rights to use, copy, modify, merge, publish, distribute, sublicense, 
***  and/or sell copies of the Software, and to permit persons to whom the 
***  Software is furnished to do so, subject to the following conditions:
***
***  The above copyright notice and this permission notice shall be included
***  in all copies or substantial portions of the Software.
***
*** THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, 
*** EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES
*** OF MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. 
*** IN NO EVENT SHALL THE AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY
*** CLAIM, DAMAGES OR OTHER LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT
*** OR OTHERWISE, ARISING FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR
*** THE USE OR OTHER DEALINGS IN THE SOFTWARE.
***
***
***************************************************************************/





// FMMMclass needs to be #defined in the #including file

void FMMMclass::updateAffectedModules()
// wrt to _movingModuleIdx, _moveFromPart and _moveToPart
{
  unsigned newLockedIds= 1<<_moveToPart;
  const HGFNode& n = _problem.getHGraph().getNodeByIdx(_movingModuleIdx);
  unsigned netId;

  const unsigned short * evalTallies
         = (reinterpret_cast<const NetCut2way&>(_eval)).getTallies();

  for(itHGFEdgeLocal  e = n.edgesBegin(); e != n.edgesEnd(); 
       e++,  _eval.moveModuleOnNet(netId,_moveFromPart, _moveToPart) )
  {
     const HGFEdge& net=(**e);
     netId=net.getIndex();

     if (net.getDegree()==2)
      {
          itHGFNodeLocal node=net.nodesBegin();
          unsigned nodeIdx;
          if ( (*node )->getIndex() == _movingModuleIdx)
               nodeIdx = (*++node)->getIndex();
          else nodeIdx = (*  node)->getIndex();
          if(!_lockedModules[nodeIdx]) 
          {
             unsigned curPart = (*_curPart)[nodeIdx].isInPart(0) ? 0 : 1;
             if (curPart==_moveFromPart)
                   {   updateGainOfModule(nodeIdx,UINT_MAX,1-curPart,2);      }
             else  {   updateGainOfModule(nodeIdx,UINT_MAX,_moveFromPart,-2); }
          }
          continue;
     }

     if (_lockedNets[netId]) continue;
     if (_lockedNetConfigIds[netId] & (~newLockedIds)) _lockedNets[netId]=true;
     else _lockedNetConfigIds[netId] |=  newLockedIds; 

     if(_eval.netCostNotAffectedByMove(netId,_moveFromPart,_moveToPart)) 
     continue;

//   {
          const unsigned short * _tallies=evalTallies+2*netId;
          itHGFNodeLocal nodeIter=net.nodesBegin();   // 3+ nodes on the net


          if ( _tallies[_moveToPart  ]==0 )
          {
             for(;nodeIter!=net.nodesEnd();nodeIter++) 
             {  
                unsigned nodeIdx=(*nodeIter)->getIndex();
                if (_lockedModules[nodeIdx]) continue;
                updateGainOfModule(nodeIdx,UINT_MAX, _moveToPart, 1); 
             }
             continue;
          }

          if ( _tallies[_moveFromPart]==1 )
          {
             for(;nodeIter!=net.nodesEnd();nodeIter++) 
             {  
                unsigned nodeIdx=(*nodeIter)->getIndex();
                if (_lockedModules[nodeIdx]) continue;
                updateGainOfModule(nodeIdx, UINT_MAX, _moveFromPart,-1); 
             }
             continue;
          }


          if ( net.getDegree()==3)
            for(;nodeIter!=net.nodesEnd();nodeIter++) // 
            {
               unsigned nodeIdx=(*nodeIter)->getIndex();
               if (_lockedModules[nodeIdx]) continue;

               if ( (*_curPart)[nodeIdx].isInPart(_moveFromPart) )
               {  
                  if (_tallies[_moveFromPart]==2) 
                     updateGainOfModule(nodeIdx, UINT_MAX, _moveToPart, 1);
               }
               else // net was uncut
               {
                  if ( _tallies[_moveToPart]  ==1)
                    updateGainOfModule(nodeIdx, UINT_MAX, _moveFromPart,-1);
               }
            }
          else
           for(;nodeIter!=net.nodesEnd();nodeIter++) // 4+ nodes
            {
               unsigned nodeIdx=(*nodeIter)->getIndex();
               if (_lockedModules[nodeIdx]) continue;

               if ( (*_curPart)[nodeIdx].isInPart(_moveFromPart) )
               {
                  if (_tallies[_moveFromPart]==2)
                  {
                     updateGainOfModule(nodeIdx, UINT_MAX, _moveToPart, 1); 
                     break;
                  }
               }
               else if ( _tallies[_moveToPart]  ==1)
               {
                    updateGainOfModule(nodeIdx, UINT_MAX, _moveFromPart,-1);
                    break;
               }
            }
          continue; 
//   }
 }
}
