/**************************************************************************
***    
*** Copyright (c) 1995-2000 Regents of the University of California,
***               Andrew E. Caldwell, Andrew B. Kahng and Igor L. Markov
***
***  Contact author(s): abk@cs.ucla.edu, imarkov@cs.ucla.edu
***  Affiliation:   UCLA, Computer Science Department,
***                 Los Angeles, CA 90095-1596 USA
***
***  Permission is hereby granted, free of charge, to any person obtaining 
***  a copy of this software and associated documentation files (the
***  "Software"), to deal in the Software without restriction, including
***  without limitation 
***  the rights to use, copy, modify, merge, publish, distribute, sublicense, 
***  and/or sell copies of the Software, and to permit persons to whom the 
***  Software is furnished to do so, subject to the following conditions:
***
***  The above copyright notice and this permission notice shall be included
***  in all copies or substantial portions of the Software.
***
*** THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, 
*** EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES
*** OF MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. 
*** IN NO EVENT SHALL THE AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY
*** CLAIM, DAMAGES OR OTHER LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT
*** OR OTHERWISE, ARISING FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR
*** THE USE OR OTHER DEALINGS IN THE SOFTWARE.
***
***
***************************************************************************/




//created by Andrew Caldwell and Igor Markov on 05/25/98

#ifdef _MSC_VER
#pragma warning(disable:4786)
#endif

#include "baseGainBucket.h"

void BaseBucketArray::removeAll()
{
    for(unsigned g = 0; g < size(); g++)
    {
	GenericGainElement* elmt = (*this)[g]._next;
	while(elmt != NULL)
	{
	    elmt->disconnect();
	    elmt = (*this)[g]._next;
	}
    }

    _highestNonEmpty = 0;
    resetAfterGainUpdate();
}
bool BaseBucketArray::isConsistent()
{
  bool consistent = true;
  for(unsigned g = 0; g < size(); g++)
 	if(!(*this)[g].isConsistent())
	    consistent = false;

  return consistent;
}

//NOTE: gain offset is the offset that gives use the bucket
//coresponding to gain ==0.  Note that this is a vector
//of size 2*_gainOffset+1.
void BaseBucketArray::setupForClip()
{
    int g;
    for(g = _gainOffset-1; g >= 0; g--)
	(*this)[_gainOffset].insertBehind( (*this)[g] );
    for(g = _gainOffset+1; g < static_cast<int>(this->size()); g++)
	(*this)[_gainOffset].insertAhead( (*this)[g] );

    GenericGainElement* e;
    for(e = (*this)[_gainOffset]._next; e != NULL; e = e->_next)
    {
	e->_gainOffset = e->_gain;
	e->_gain = 0;
    }
}

ostream& operator<<(ostream& os, const BaseBucketArray& bucket)
{
    os<<" BaseBucketArray:"<<endl;
    os<<"  MaxGain:  "<<bucket._gainOffset<<endl;

    for(int g = bucket.size()-1; g >= 0; g--)
	os<<g - bucket._gainOffset <<") "<<bucket[g];

    os<<endl;
    return os;
}
