/**************************************************************************
***    
*** Copyright (c) 1995-2000 Regents of the University of California,
***               Andrew E. Caldwell, Andrew B. Kahng and Igor L. Markov
***
***  Contact author(s): abk@cs.ucla.edu, imarkov@cs.ucla.edu
***  Affiliation:   UCLA, Computer Science Department,
***                 Los Angeles, CA 90095-1596 USA
***
***  Permission is hereby granted, free of charge, to any person obtaining 
***  a copy of this software and associated documentation files (the
***  "Software"), to deal in the Software without restriction, including
***  without limitation 
***  the rights to use, copy, modify, merge, publish, distribute, sublicense, 
***  and/or sell copies of the Software, and to permit persons to whom the 
***  Software is furnished to do so, subject to the following conditions:
***
***  The above copyright notice and this permission notice shall be included
***  in all copies or substantial portions of the Software.
***
*** THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, 
*** EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES
*** OF MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. 
*** IN NO EVENT SHALL THE AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY
*** CLAIM, DAMAGES OR OTHER LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT
*** OR OTHERWISE, ARISING FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR
*** THE USE OR OTHER DEALINGS IN THE SOFTWARE.
***
***
***************************************************************************/




//created by Andrew Caldwell and Igor Markov on 09/19/98

#ifndef __SPLIT_GAIN_BUCKET_H__
#define __SPLIT_GAIN_BUCKET_H__

#include <vector>
using std::vector;
#include "ABKCommon/abkcommon.h"
#include "baseGainBucket.h"

unsigned const kSplitGainBucketFactor=16;

inline int splitBucketFunction(int initialGain, int updatedGain)
{
   return kSplitGainBucketFactor*max(0,updatedGain)+(initialGain+updatedGain);
}

class SplitBucketArray : public BaseBucketArray
{
  public:
    SplitBucketArray() : BaseBucketArray() {}

    ~SplitBucketArray(){}

    void setMaxGain(unsigned maxG);

    void push(GenericGainElement& elmt);

};

//____________________IMPLEMENTATIONS__________________________//


inline void SplitBucketArray::setMaxGain(unsigned maxG)
{ 	
    _gainOffset = maxG * (kSplitGainBucketFactor+2);
    _maxBucketIdx =  2 * _gainOffset;
    this->insert(end(), _maxBucketIdx+1-size(), GainList());
    _highestNonEmpty = _highestValid = 0;
}

inline void SplitBucketArray::push(GenericGainElement& elmt)
{
    abkassert(_maxBucketIdx != -1, 
	"must call setMaxGain before using the SplitBucketArray");

    int bucket = splitBucketFunction(elmt._gainOffset,elmt._gain);
        bucket = max(min(_maxBucketIdx,  bucket+_gainOffset), 0);
    (*this)[bucket].push(elmt);
    _highestNonEmpty = max(_highestNonEmpty, bucket);
}

#endif

