/**************************************************************************
***    
*** Copyright (c) 1995-2000 Regents of the University of California,
***               Andrew E. Caldwell, Andrew B. Kahng and Igor L. Markov
***
***  Contact author(s): abk@cs.ucla.edu, imarkov@cs.ucla.edu
***  Affiliation:   UCLA, Computer Science Department,
***                 Los Angeles, CA 90095-1596 USA
***
***  Permission is hereby granted, free of charge, to any person obtaining 
***  a copy of this software and associated documentation files (the
***  "Software"), to deal in the Software without restriction, including
***  without limitation 
***  the rights to use, copy, modify, merge, publish, distribute, sublicense, 
***  and/or sell copies of the Software, and to permit persons to whom the 
***  Software is furnished to do so, subject to the following conditions:
***
***  The above copyright notice and this permission notice shall be included
***  in all copies or substantial portions of the Software.
***
*** THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, 
*** EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES
*** OF MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. 
*** IN NO EVENT SHALL THE AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY
*** CLAIM, DAMAGES OR OTHER LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT
*** OR OTHERWISE, ARISING FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR
*** THE USE OR OTHER DEALINGS IN THE SOFTWARE.
***
***
***************************************************************************/




//created by Andrew Caldwell on 02/02/99

#ifdef _MSC_VER
#pragma warning(disable:4786)
#endif

#include "hgWDimsFromDB.h"

//most of this code is copied from the HGfromDB ctor

HGraphWDimensionsFromDB::HGraphWDimensionsFromDB(const DB& db)
    :HGraphWDimensions(db.getNetlist().getNumCells(), 1) 
{
    const dbNetlist& netlist = db.getNetlist();

    unsigned numCells = netlist.getNumCells();
    if (db.spatial.constraints.hasFGC())
      _numTerminals=db.spatial.constraints.getFGC().getSize();
    else _numTerminals=0;

    _nodeNames = vector<const char*>(getNumNodes());

    for(unsigned i=0; i < numCells; i++)
    {
        const dbCell &cell=netlist.getCellByIdx(i);
	_nodeWidths[i]  = cell.getWidth();
	_nodeHeights[i] = cell.getHeight();
	_nodeSymmetries[i] = cell.getMaster().getSymmetry();
        _nodes[i]->setWeight(_nodeHeights[i] * _nodeWidths[i]);

        char* tmpName;
        if(cell.getName() == NULL)
        {
            tmpName = new char[9];
            sprintf(tmpName, "cell%d", i);
        }
        else
        {
            tmpName = new char[strlen(cell.getName())+1];
            strcpy(tmpName, cell.getName());
        }
        _nodeNames[i] = tmpName;
        _namesMap[tmpName] = i;
    }

    itNetGlobal nItr;
    for(nItr = netlist.netsBegin(); nItr != netlist.netsEnd(); nItr++)
    {
        HGFEdge* edge=addEdge();

        abkassert(edge->getIndex()==(*nItr)->getIndex(),
        			"netlist traversed out of order");
	abkassert(edge->getWeight() > 0, "constructed 0 weight edge");

	const dbNet& net = **nItr;

        itCellLocal c;
        for(c=(*nItr)->sourcesBegin(); c!=(*nItr)->sourcesEnd(); c++)
	{
	    const dbCell &cell=**c;
	    const dbPin*  pin  = cell.commonPin(&net);
	    abkassert(pin != NULL, "null commonPin return");

	    addSrcWPin(getNodeByIdx(cell.getIndex()), *edge, 
					pin->getMaster().getCenter());
	}

        for(c=(*nItr)->sourceSinksBegin(); c!=(*nItr)->sourceSinksEnd(); c++)
	{
	    const dbCell &cell=**c;
	    const dbPin*  pin  = cell.commonPin(&net);
	    abkassert(pin != NULL, "null commonPin return");

	    addSrcSnkWPin(getNodeByIdx(cell.getIndex()), *edge,
					pin->getMaster().getCenter());
	}

        for(c=(*nItr)->sinksBegin(); c!=(*nItr)->sinksEnd(); c++)
	{
	    const dbCell &cell=**c;
	    const dbPin*  pin  = cell.commonPin(&net);
	    abkassert(pin != NULL, "null commonPin return");

	    addSnkWPin(getNodeByIdx(cell.getIndex()), *edge,
					pin->getMaster().getCenter());
	}
    }

    finalize();
    sortAsDB();
}

