/**************************************************************************
***    
*** Copyright (c) 1995-2000 Regents of the University of California,
***               Andrew E. Caldwell, Andrew B. Kahng and Igor L. Markov
***
***  Contact author(s): abk@cs.ucla.edu, imarkov@cs.ucla.edu
***  Affiliation:   UCLA, Computer Science Department,
***                 Los Angeles, CA 90095-1596 USA
***
***  Permission is hereby granted, free of charge, to any person obtaining 
***  a copy of this software and associated documentation files (the
***  "Software"), to deal in the Software without restriction, including
***  without limitation 
***  the rights to use, copy, modify, merge, publish, distribute, sublicense, 
***  and/or sell copies of the Software, and to permit persons to whom the 
***  Software is furnished to do so, subject to the following conditions:
***
***  The above copyright notice and this permission notice shall be included
***  in all copies or substantial portions of the Software.
***
*** THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, 
*** EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES
*** OF MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. 
*** IN NO EVENT SHALL THE AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY
*** CLAIM, DAMAGES OR OTHER LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT
*** OR OTHERWISE, ARISING FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR
*** THE USE OR OTHER DEALINGS IN THE SOFTWARE.
***
***
***************************************************************************/




#include <iostream.h>
#include "partitioning.h"
#include "partEvals.h"

int main(int argc, char *argv[]) 
{
    cout<<endl<<"NetCut Evaluator"<<endl<<endl;

    StringParam  baseFileName("f",argc,argv);
    BoolParam    help1("h",    argc, argv);
    BoolParam    help2("help", argc, argv);
    NoParams     noParams(     argc, argv);

    if(help1.found() || help2.found() || noParams.found())
    {
	cout<<" -f    <aux filename>"<<endl;
	return 0;
    }

    abkfatal(baseFileName.found()," -f baseFileName not found");

    PartitioningProblem::Parameters pParams(Verbosity("0_0_0"));
    PartitioningProblem problem(baseFileName, pParams);

    Partitioning part=problem.getSolnBuffers()[0];

    UnivPartEval eval=PartEvalType::NetCutWNetVec;

    vector<double> partAreas(problem.getNumPartitions(), 0);
    bool completeSoln = true;
    problem.propagateTerminals(); 

	//since this is being released as an executable, it
	//will only operate on PartProbs loaded from files.
	//Such problems always have exactly 1 solution...
    const Partitioning& soln = problem.getSolnBuffers()[0];
cout<<soln<<endl;

    for(itHGFNodeGlobal n = problem.getHGraph().nodesBegin();
	n != problem.getHGraph().nodesEnd(); n++)
    {
	if(soln[(*n)->getIndex()].numberPartitions() != 1)
	{
		completeSoln = false;
		cout<<" Node "<<(*n)->getIndex()<<"  "
		    <<soln[(*n)->getIndex()]<<endl;
	}
	else
		partAreas[soln[(*n)->getIndex()].lowestNumPart()] +=
			(*n)->getWeight();
    }


    if(completeSoln)
    {
        cout<<"NetCut     : "<<eval.computeCost(problem,part)<<endl;
	cout<<"Part Areas : "<<partAreas;
    } 
    else
    {
	cout<<" The solution is not complete - at least one node\n"
	    <<" is either more than one partition, or none"<<endl;
     }

    return 0;
}
