/**************************************************************************
***    
*** Copyright (c) 1995-2000 Regents of the University of California,
***               Andrew E. Caldwell, Andrew B. Kahng and Igor L. Markov
***
***  Contact author(s): abk@cs.ucla.edu, imarkov@cs.ucla.edu
***  Affiliation:   UCLA, Computer Science Department,
***                 Los Angeles, CA 90095-1596 USA
***
***  Permission is hereby granted, free of charge, to any person obtaining 
***  a copy of this software and associated documentation files (the
***  "Software"), to deal in the Software without restriction, including
***  without limitation 
***  the rights to use, copy, modify, merge, publish, distribute, sublicense, 
***  and/or sell copies of the Software, and to permit persons to whom the 
***  Software is furnished to do so, subject to the following conditions:
***
***  The above copyright notice and this permission notice shall be included
***  in all copies or substantial portions of the Software.
***
*** THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, 
*** EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES
*** OF MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. 
*** IN NO EVENT SHALL THE AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY
*** CLAIM, DAMAGES OR OTHER LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT
*** OR OTHERWISE, ARISING FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR
*** THE USE OR OTHER DEALINGS IN THE SOFTWARE.
***
***
***************************************************************************/




// Created by Andrew Caldwell on 05/14/99
//  Based on netStacks.cxx by Igor Markov

#ifdef _MSC_VER
#pragma warning(disable:4786)
#endif

#include <iostream.h>
#include "spNetStacks.h"

SmallPlacerNetStacks::SmallPlacerNetStacks
				(const SmallPlacementProblem& problem,
				 double minWL)
	: _numNets(problem.getNumNets()), _totalWidth(0),
	  _bloatWidths(problem.getNumCells()), _minWL(minWL)
{
    unsigned numCells   = problem.getNumCells();
    unsigned maxNumPins = _numNets*(numCells+1);

    _netData   = new XInterval[maxNumPins];	//net costs
    _netStacks = new XInterval*[_numNets];	//pointers to current net costs

    _adjNets   = new EdgeAndOffset[maxNumPins];
    _firstNets = new EdgeAndOffset*[numCells+1];

    _cost.reserve(numCells+2);

    unsigned c;
    for(c = 0; c != problem.getNumCells(); c++)
        _totalWidth += problem.getCellWidth(c);

    const SmallPlacementRow& row = problem.getRows()[0];
    _leftEdge  = row.xMin;
    _rightEdge = row.xMax;

    double space = _rightEdge - _leftEdge;
    double bloat = (space-_totalWidth)/numCells;

    for(c = 0; c < problem.getNumCells(); c++)
        _bloatWidths[c] = problem.getCellWidth(c)+bloat;

    populate(problem);
}

void SmallPlacerNetStacks::populate(const SmallPlacementProblem& problem)
{
    double   initCost = 0;
    unsigned nextEdgeStack = 0;

    XInterval rowIvl(problem.getRows()[0].xMin,problem.getRows()[0].xMax);

    unsigned n;
    for(n = 0; n < problem.getNumNets(); n++)
    {
	_netStacks[n] = _netData+nextEdgeStack;		
	nextEdgeStack += problem.getNumCells();

	const BBox& netBBox = problem.getNetTerminalBBox(n);
	XInterval& netIvl = *_netStacks[n];

	netIvl = XInterval(netBBox.xMin, netBBox.xMax);

	if ( ! netIvl.isEmpty() )
	{
            double  rmin=rowIvl.min;
            double& nmax=netIvl.max;
	    if ( nmax < rmin ) nmax = rmin; //same as += rowInv.min
            else if ( netIvl.min > rowIvl.max )
               {netIvl.min = rowIvl.max; }//same as += rowInv.max

	    initCost += max(netIvl.getLength(), _minWL);
	}
    }
    _cost.push_back(initCost);

    const SmallPlacementNetlist& netlist = problem.getNetlist();

    unsigned c, idx = 0;
    for(c = 0; c < problem.getNumCells(); c++)
    {
        _firstNets[c] = _adjNets+idx;
        for(n = 0; n < problem.getNumNets(); n++)
   	{
	    double pinOffset = netlist.getPinOffset(c,n).x;

            if(pinOffset != DBL_MAX)	//they are connected...
	    {
                _adjNets[idx].edge = n;
                _adjNets[idx].pinOffset = pinOffset;
		idx++;
	    }
	}
    }
    _firstNets[c] = _adjNets+idx;
}

SmallPlacerNetStacks::~SmallPlacerNetStacks()
{
    delete[] _netData;
    delete[] _netStacks;
    delete[] _adjNets;
    delete[] _firstNets;
}

void SmallPlacerNetStacks::print()
{
    cout<<"Current WL for each net is:"<<endl;

    unsigned n;
    for(n = 0; n < _numNets; n++)
	cout<<setw(2)<<n<<": "<<_netStacks[n]->getLength()<<endl;
}

