/**************************************************************************
***
***    Copyright (c) 1995-1997 by ABKGroup, UCLA VLSI CAD Laboratory,
***    UCLA Computer Science Department, Los Angeles, CA 90095-1596 USA,
***    and by the Regents of the University of California.
***    All rights reserved.
***
***    No portion of this file may be used, copied, or transmitted for
***    any purpose, including any research or commercial purpose,
***    without the express written consent of Professor Andrew B. Kahng,
***    UCLA Computer Science Department, 3713 Boelter Hall, Los Angeles,
***    CA 90095-1596 USA.   Any copy made of this file must include this
***    notice.    ABKGroup and the Regents of the University of California
***    retain title to all copies of this file and/or its compiled
***    derivatives.
***
***************************************************************************/


#include "partProbForCapo.h"
#include "capoBlock.h"
#include "subHGForCapo.h"
#include "termiProp.h"
#include "capoPlacer.h"

PartitioningProblemForCapo::PartitioningProblemForCapo(
		const CapoBlock&		block,
		const CapoPlacer&		capo,
		      bool                      isHCut,
		      double                    splitPt,	
		      Verbosity                 verb)
	: PartitioningProblem(), 
	  _hgraphOfNetlist(capo.getNetlistHGraph()), 
	  _cellToBlockMap (capo.getCellToBlockMap()),
          _capoPl(capo), _verb(verb)
{
    _blocks = vector<const CapoBlock*>(1);
    _blocks[0] = &block;

    initToNULL();

    double cellArea = block.getTotalCellArea();
    _capacities = new vector<vector<double> >
			(2,vector<double>(1,cellArea/2.0));
    _totalWeight = new vector<double>(1, cellArea);
    _maxCapacities=new vector<vector<double> >(2, vector<double>(1,0));
    _minCapacities=new vector<vector<double> >(2, vector<double>(1,0));
    setTolerance(0);

	//partitions get set only so we can construct the HGraph
	//and do termi-prop..it is expected that someone
	//will set the capacities, etc. before use
    const BBox& blockBBox = block.getBBox();
    _partitions = new vector<BBox>(2,BBox());
    if(isHCut)
    {
	_nDim = 2; 
	_mDim = 1;
        (*_partitions)[0] = BBox(blockBBox.xMin,splitPt,
                              blockBBox.xMax,blockBBox.yMax);
        (*_partitions)[1] = BBox(blockBBox.xMin,blockBBox.yMin,
                              blockBBox.xMax,splitPt);
    }
    else
    {
	_nDim = 1; 
	_mDim = 2;
   	(*_partitions)[0] = BBox(blockBBox.xMin, blockBBox.yMin,
                              splitPt, blockBBox.yMax);
    	(*_partitions)[1] = BBox(splitPt, blockBBox.yMin,
                              blockBBox.xMax, blockBBox.yMax);
    }

    _setPartitionCenters();

    buildHGraph();

    setBuffer();

    _ownsData = false;
}

PartitioningProblemForCapo::PartitioningProblemForCapo(
                const vector<const CapoBlock*>&        blocks,
                const HGraphFixed&              hgraphOfNetlist,
                const CapoPlacer&               capo,
                const vector<const CapoBlock*>& cellToBlockMap,
                Verbosity verb)
        : PartitioningProblem(), _blocks(blocks),
          _hgraphOfNetlist(hgraphOfNetlist), 
	  _cellToBlockMap(cellToBlockMap),
          _capoPl(capo), _hCutWSMethod(UINT_MAX),
          _optimisticAdj(UINT_MAX), _verb(verb)
{
    abkfatal(_blocks.size() == 2, "can only build problems for 2 blocks");

    initToNULL();

    setPartDims(); //use the block shapes/capacities for the partitions

    buildHGraph();

    setBuffer();

    _ownsData = false;
}

PartitioningProblemForCapo::~PartitioningProblemForCapo()
{
        if(_hgraph != NULL)
        {
            delete _subHG;
            _hgraph = _subHG = NULL;
        }

        if(_fixedConstr != NULL)
        {
            delete _fixedConstr;
            _fixedConstr = NULL;
        }

        if(_solnBuffers != NULL)
        {
            delete _solnBuffers;
            _solnBuffers = NULL;
        }

        if(_capacities != NULL)
        {
            delete _capacities;
            _capacities = NULL;
        }

        if(_partitions != NULL)
        {
            delete _partitions;
            _partitions = NULL;
        }

        if(_padBlocks != NULL)
        {
            delete _padBlocks;
            _padBlocks = NULL;
        }
        if(_terminalToBlock != NULL)
        {
            delete _terminalToBlock;
            _terminalToBlock = NULL;
        }
}


